package java.nio.file.attribute;

import java.util.EnumSet;
import java.util.Iterator;
import java.util.HashSet;
import java.util.Collections;
import java.util.Set;

public final class AclEntry
{
    private final AclEntryType type;
    private final UserPrincipal who;
    private final Set<AclEntryPermission> perms;
    private final Set<AclEntryFlag> flags;
    private volatile int hash;
    
    private AclEntry(AclEntryType type, UserPrincipal who, Set<AclEntryPermission> perms, Set<AclEntryFlag> flags) {
        this.type = type;
        this.who = who;
        this.perms = perms;
        this.flags = flags;
    }
    
    public static AclEntry.Builder newBuilder() {
        return new AclEntry.Builder(null, null, Collections.emptySet(), Collections.emptySet());
    }
    
    public static AclEntry.Builder newBuilder(AclEntry aclEntry) {
        return new AclEntry.Builder(aclEntry.type, aclEntry.who, aclEntry.perms, aclEntry.flags);
    }
    
    public AclEntryType type() {
        return this.type;
    }
    
    public UserPrincipal principal() {
        return this.who;
    }
    
    public Set<AclEntryPermission> permissions() {
        return new HashSet(this.perms);
    }
    
    public Set<AclEntryFlag> flags() {
        return new HashSet(this.flags);
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if (o == null || !(o instanceof AclEntry)) {
            return false;
        }
        AclEntry aclEntry = (AclEntry)o;
        return this.type == aclEntry.type && this.who.equals(aclEntry.who) && this.perms.equals(aclEntry.perms) && this.flags.equals(aclEntry.flags);
    }
    
    private static int hash(int n, Object o) {
        return n * 127 + o.hashCode();
    }
    
    @Override
    public int hashCode() {
        if (this.hash != 0) {
            return this.hash;
        }
        return this.hash = hash(hash(hash(this.type.hashCode(), this.who), this.perms), this.flags);
    }
    
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append(this.who.getName());
        sb.append(':');
        Iterator<AclEntryPermission> iterator = this.perms.iterator();
        while (iterator.hasNext()) {
            sb.append(((AclEntryPermission)iterator.next()).name());
            sb.append('/');
        }
        sb.setLength(sb.length() - 1);
        sb.append(':');
        if (!this.flags.isEmpty()) {
            Iterator<AclEntryFlag> iterator2 = this.flags.iterator();
            while (iterator2.hasNext()) {
                sb.append(((AclEntryFlag)iterator2.next()).name());
                sb.append('/');
            }
            sb.setLength(sb.length() - 1);
            sb.append(':');
        }
        sb.append(this.type.name());
        return sb.toString();
    }
    
    AclEntry(AclEntryType aclEntryType, UserPrincipal userPrincipal, Set set, Set set2, AclEntry.1 object) {
        this(aclEntryType, userPrincipal, set, set2);
    }
    
    public static final class Builder
    {
        private AclEntryType type;
        private UserPrincipal who;
        private Set<AclEntryPermission> perms;
        private Set<AclEntryFlag> flags;
        
        private Builder(AclEntryType type, UserPrincipal who, Set<AclEntryPermission> perms, Set<AclEntryFlag> flags) {
            assert perms != null && flags != null;
            this.type = type;
            this.who = who;
            this.perms = perms;
            this.flags = flags;
            return;
        }
        
        public AclEntry build() {
            if (this.type == null) {
                throw new IllegalStateException("Missing type component");
            }
            if (this.who == null) {
                throw new IllegalStateException("Missing who component");
            }
            return new AclEntry(this.type, this.who, this.perms, this.flags);
        }
        
        public AclEntry.Builder setType(AclEntryType type) {
            if (type == null) {
                throw new NullPointerException();
            }
            this.type = type;
            return this;
        }
        
        public AclEntry.Builder setPrincipal(UserPrincipal who) {
            if (who == null) {
                throw new NullPointerException();
            }
            this.who = who;
            return this;
        }
        
        private static void checkSet(Set<?> set, Class<?> clazz) {
            for (Object next : set) {
                if (next == null) {
                    throw new NullPointerException();
                }
                clazz.cast(next);
            }
        }
        
        public AclEntry.Builder setPermissions(Set<AclEntryPermission> c) {
            Set perms;
            if (c.isEmpty()) {
                perms = Collections.emptySet();
            }
            else {
                perms = EnumSet.copyOf(c);
                checkSet(perms, AclEntryPermission.class);
            }
            this.perms = perms;
            return this;
        }
        
        public AclEntry.Builder setPermissions(AclEntryPermission... array) {
            EnumSet<AclEntryPermission> none = EnumSet.noneOf(AclEntryPermission.class);
            int length = array.length;
            int i = 0;
            while (i < length) {
                AclEntryPermission aclEntryPermission = array[i];
                if (aclEntryPermission == null) {
                    throw new NullPointerException();
                }
                none.add(aclEntryPermission);
                ++i;
            }
            this.perms = none;
            return this;
        }
        
        public AclEntry.Builder setFlags(Set<AclEntryFlag> c) {
            Set flags;
            if (c.isEmpty()) {
                flags = Collections.emptySet();
            }
            else {
                flags = EnumSet.copyOf(c);
                checkSet(flags, AclEntryFlag.class);
            }
            this.flags = flags;
            return this;
        }
        
        public AclEntry.Builder setFlags(AclEntryFlag... array) {
            EnumSet<AclEntryFlag> none = EnumSet.noneOf(AclEntryFlag.class);
            int length = array.length;
            int i = 0;
            while (i < length) {
                AclEntryFlag aclEntryFlag = array[i];
                if (aclEntryFlag == null) {
                    throw new NullPointerException();
                }
                none.add(aclEntryFlag);
                ++i;
            }
            this.flags = none;
            return this;
        }
    }
}
